// This class is used to handle which bluetooth communication class we use.

public class RBCBlue extends Thread {
	// The bluetooth communication object which can be either BluetoothServer, or BluetoothServerEasyBlue.
	private BluetoothComm 		server;
	private RobotTracker 		rb;					// Reference to the main program

	public RBCBlue(RobotTracker mainApp) {
		rb = mainApp;
		if (!rb.USE_EASY_BLUE) {
			server = new BluetoothServer(rb);
		} else {
			server = new BluetoothServerEasyBlue(rb);
		}
	}

	// Set the URL of the robot to connect to
	public void setURL(String url) {
		if (rb.USE_EASY_BLUE) {
			((BluetoothServerEasyBlue)server).setURL(url);
		}
	}

	// Send data to the robot
	public void sendData(byte[] data) {
		server.sendData(data, true);
	}

	// Send debug data to the robot
	public void sendDebugData(byte[] data) {
		server.sendDebugData(data);
	}

	// Send data to the robot but without any preparation (no command is automatically added to the beginning).
	public void sendSpecificData(byte[] data) {
		server.sendData(data, false);
	}


	// Start running RBCBlue in a thread and initialize the server
	public void run() {
		try {
			if (server.connectToRobot()) {
				rb.showRobotConnected();
				while (!rb.tracking);
				server.startServer();
			}
		}
		catch (Exception e) {
			System.err.println(e.toString());
            System.err.println(e.getStackTrace());
            e.printStackTrace();
		}
	}
}