import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

// Scanner window that allows you to scan for near bluetooth devices.
public class ScanDialog extends JDialog {
	private JList<String> 			deviceList;		// Displays the list of bluetooth devices found
	private JTable 					configTable;	// Table containing configuration data
	private BluetoothDeviceFinder 	deviceFinder;	// Class used to find near bluetooth devices;
	private JTextField 				statusPane;		// Status bar on the window
	private JButton 				newButton;		// Make new entry into table
	private JButton 				removeButton;	// remove selected entry from table
	private JButton 				scanButton;		// Starts the process of scanning for nearby bluetooth devices
	private JButton 				addButton;		// Add a bluetooth address to the list of robot addresses

	private	TreeMap<Integer, String> robotAddresses;

	public ScanDialog(JFrame parent, TreeMap<Integer, String> initAddresses) {
		super(parent, "Configure Robot Bluetooth Devices", true);
		robotAddresses = initAddresses;

		deviceFinder = new BluetoothDeviceFinder();
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		GridBagConstraints c = new GridBagConstraints();

		// Add the labels
		JLabel label = new JLabel("Configured Robot Addresses");
		c.gridx = 0; c.gridy = 0;
		c.gridwidth = 2; c.gridheight = 1;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(label, c);
		add(label);
        //add(label, c);

		label = new JLabel("Detected Bluetooth Devices (Robot's look like: 0017A0016xxx)");
		c.gridx = 2;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(label, c);
		add(label);
        //add(label, c);


		// Add the configuration table
		JScrollPane scrollPane = new JScrollPane();
		configTable = new JTable(new BlueAddressTableModel());
		configTable.getColumnModel().getColumn(0).setPreferredWidth(15);
		configTable.getColumnModel().getColumn(0).setResizable(false);
		configTable.getColumnModel().setColumnSelectionAllowed(false);
		configTable.setDragEnabled(false);
		configTable.setCellSelectionEnabled(false);
		configTable.setRowSelectionAllowed(true);
		configTable.setAutoCreateRowSorter(false);
		configTable.getTableHeader().setReorderingAllowed(false);
		configTable.getTableHeader().setResizingAllowed(false);
		configTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scrollPane.setViewportView(configTable);
		c.gridx = 0; c.gridy = 1;
		c.gridwidth = 2; c.gridheight = 1;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1; c.weighty = 1;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(scrollPane, c);
		add(scrollPane);

		// Add the device list
		deviceList = new JList<String>();
        deviceList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        scrollPane = new JScrollPane(deviceList, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
            						 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		c.gridx = 2; c.gridy = 1;
		c.gridwidth = 2; c.gridheight = 1;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1; c.weighty = 1;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(scrollPane, c);
		add(scrollPane);

		// Add the New button
		newButton = new JButton("New");
		newButton.setEnabled(true);
		newButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				handleNewButton();
			}
		});
		c.gridx = 0; c.gridy = 2;
		c.gridwidth = 1; c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(newButton, c);
		add(newButton);

		// Add the Remove button
		removeButton = new JButton("Remove");
		removeButton.setEnabled(true);
		removeButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				handleRemoveButton();
			}
		});
		c.gridx = 1; c.gridy = 2;
		c.gridwidth = 1; c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(removeButton, c);
		add(removeButton);

		// Add the Scan button
		scanButton = new JButton("Scan");
		scanButton.setEnabled(true);
		scanButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				handleScanButton();
			}
		});
		c.gridx = 2; c.gridy = 2;
		c.gridwidth = 1; c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(scanButton, c);
		add(scanButton);

		// Add the Add Selection button
		addButton = new JButton("Add");
		addButton.setEnabled(false);
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				handleAddButton();
			}
		});
		c.gridx = 3; c.gridy = 2;
		c.gridwidth = 1; c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(addButton, c);
		add(addButton);

		// Add the status panel
		statusPane = new JTextField("Press Scan to search for the robot's address.  Scan once with robot off, then with it on to identify address");
        statusPane.setEditable(false);
        statusPane.setBackground(Color.black);
        statusPane.setForeground(Color.white);
        statusPane.setPreferredSize(new Dimension(580,20));
		c.gridx = 0; c.gridy = 3;
		c.gridwidth = 4; c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1; c.weighty = 0;
		c.insets = new Insets(2, 2, 2, 2);
		layout.setConstraints(statusPane, c);
		add(statusPane);

		// Set up the event handler for window closing
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				closeWindow();
			}
		});

		// Load the input settings into the table
		Object[] keys = robotAddresses.keySet().toArray();
		for (int i=0;i<keys.length;i++) {
			int id = ((Integer)keys[i]).intValue();
			String value = (String)robotAddresses.get(id);
			((BlueAddressTableModel)configTable.getModel()).addValue(id, value);
		}

		// Set the window size
		setSize(600, 270);
		setLocationRelativeTo(getOwner());
        setResizable(false);
	}


	// Called when window is closed. Saves changes and disposes of the window.
	private void closeWindow() {
		// Check for valid settings
		TableModel model = configTable.getModel();
		int rows = configTable.getModel().getRowCount();
		for (int i=0;i<rows;i++) {
			Integer id = (Integer)model.getValueAt(i, 0);
			String value = (String)model.getValueAt(i, 1);
			robotAddresses.put(id, value);
		}
		// Save the settings
		((RobotTracker)this.getOwner()).saveConfigFile();
		setVisible(false);
	}


	// Called when new button pressed ... adds new entry into table.
	private void handleNewButton() {
		String input = JOptionPane.showInputDialog(this, "Enter the robot's id:");
		if (input != null && input != "") {
			try {
				int id = Integer.parseInt(input);
				((BlueAddressTableModel)configTable.getModel()).addValue(id, "000000000000");
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "ERROR: The robot's id must be an integer.", "Error Adding Robot", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	// Called when remove button pressed ... removes selected entry from table.
	private void handleRemoveButton() {
		if (configTable.getSelectedRow() > -1) {
			int id = ((Integer)configTable.getModel().getValueAt(configTable.getSelectedRow(), 0)).intValue();
			int n = JOptionPane.showConfirmDialog(this, "Are you sure you wish to remove robot "+id+"?", "Remove Robot Address", JOptionPane.YES_NO_OPTION);
			if (n==JOptionPane.YES_OPTION) {
				((BlueAddressTableModel)configTable.getModel()).removeValue(configTable.getSelectedRow());
				robotAddresses.remove(id);
			}
		}
	}

	// Called when scan button pressed ... starts the process of scanning for nearby bluetooth devices.
	private void handleScanButton() {
		statusPane.setText("Scanning for Bluetooth devices...");
		statusPane.update(statusPane.getGraphics());
		scan();
	}

	// Called when add button pressed ... adds the selected address from the device list as the specified robot's address.
	private void handleAddButton() {
		if (deviceList.getSelectedIndex() > -1) {
			String address = ((String)deviceList.getSelectedValue()).split(" ")[0];
			String input = JOptionPane.showInputDialog(this, "Enter the robot's id:");
			try {
				if (input != null) {
					int id = Integer.parseInt(input);
					Integer idI = new Integer(id);
					robotAddresses.put(idI, address);
					((BlueAddressTableModel)configTable.getModel()).addValue(idI, address);
	 			}
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "ERROR: The robot's id must be an integer.", "Error Adding Robot", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	// Starts scanning for nearby bluetooth devices using BluetoothDeviceFinder
	private void scan() {
		Vector<String>  urls = deviceFinder.scanForBtDevice();
        if (urls != null) {
            deviceList.setListData(urls);
            if (urls.size()<1)
            	statusPane.setText("No Bluetooth devices found.");
            else {
            	statusPane.setText("Scan complete. Select robot address above and press Add to store it in the configuration file.");
            	addButton.setEnabled(true);
			}
		}
		else {
            deviceList.setListData(new Vector<String>());	//clear the list
            statusPane.setText("No Bluetooth devices found.");
        }
        deviceList.update(deviceList.getGraphics());
        statusPane.update(statusPane.getGraphics());
	}


	// Model for the robot's address table that sets the different settings of the table.
	private class BlueAddressTableModel extends AbstractTableModel {
		// Column names of the table
		private String[] columnNames = {"Robot Id", "Bluetooth Address"};
        private Vector<String> addresses = new Vector<String>();	// List of robot addresses
        private Vector<Integer> ids = new Vector<Integer>();	// List of robot ids

        // Add a new row to the table
		public void addRow(int id, String address) {
			ids.add(new Integer(id));
			addresses.add(new String(""+address));
		}

		// Return the number of columns in the table
        public int getColumnCount() {
            return columnNames.length;
        }

        // Return the number of rows in the table
        public int getRowCount() {
            return ids.size();
        }

        // Return the specified column name at the given index
        public String getColumnName(int index) {
            return columnNames[index];
        }

        //  Add a new value to the table
		public boolean addValue(int id, String address) {
			int size = ids.size();
			for (int i=0;i<size;i++){
				if (ids.get(i).intValue()>id) {
					ids.add(i, new Integer(id));
					addresses.add(i, new String(address));
					fireTableDataChanged();
					return true;
				} else if (ids.get(i).intValue()==id) {
					return false;
				}
			}
			ids.add(new Integer(id));
			addresses.add(new String(address));
			fireTableDataChanged();
			return true;
		}

		// Return the object at a specific index.
        public Object getValueAt(int row, int col) {
            if (col == 0) {
            	return ids.get(row);
            } else {
            	return addresses.get(row);
            }
        }

        // Remove a specific value from the table
		public void removeValue(int row) {
			if (row < ids.size()) {
				ids.remove(row);
				addresses.remove(row);
				fireTableDataChanged();
			}
		}

		// Set a value at a specific index.
        public void setValueAt(Object o, int row, int col) {
        	if (col == 0) {
        		try {
        			ids.set(row, ((Integer)o));
        		} catch (Exception e) {
        			System.out.println("[ERROR] Received a non int for id.");
        		}
        	} else {
        		if (((String)o).matches("\\p{Alnum}+"))
	        		addresses.set(row, new String(""+o));
        	}
        }

        // Returns whether the cell is editable (which is true for every cell).
        public boolean isCellEditable(int row, int col) {
            return (col >= 1);
        }
    }

	public static void main(String[] args) {
		new ScanDialog(null, new TreeMap<Integer,String>()).setVisible(true);
	}
}